import React, { useContext, useState } from 'react';
import { makeStyles } from '@material-ui/core/styles';
import { Paper, Grid, FormControlLabel, Switch, Typography, Container, TextField } from '@material-ui/core';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import { useOperationsSearch, userContext } from '../../api/SwaggerApi';
import { initRequestParams } from '../../components/common/Helper';
import { useEffect } from 'react';
import { SearchBar } from '../../components/common/SearchBar/SearchBar';
import AccessControl from '../../components/auth/AccessControl';
import { Autocomplete } from '@material-ui/lab';
import { JobAsyncList } from '../../components/Job/JobAsyncList';

const useStyles = makeStyles((theme) => ({
  root: {
    width: '100%'
  },
  paper: {
    marginBottom: theme.spacing(2),
  },
  formControl: {
    marginLeft: '5px',
    marginRight: '5px',
  }
}));

export function JobListView() {
  const classes = useStyles();
  const [operations, getOperations, /*reset*/, loading] = useOperationsSearch();
  const [query, setQuery] = useState("");
  const [selectedTab, setSelectedTab] = useState(0);
  const [ownOnly, setOwnOnly] = useState(false);
  const { user } = useContext(userContext);
  const [deploymentStatus, setDeploymentStatus] = useState(null);
  const [jobList, setJobList] = useState([]);
  const [jobType, setJobType] = useState(null);

  console.log("JobListView:", operations, jobList);

  const handleTabChange = (event, tab) => {
    setSelectedTab(tab);

    if (tab === 0) {
      setDeploymentStatus(null);
    }
    else if (tab === 1) {
      setDeploymentStatus("RUNNING");
    }
    else if (tab === 2) {
      setDeploymentStatus("FINISHED");
    }
    else if (tab === 3) {
      setDeploymentStatus("QUEUED");
    }
  };

  const handleChangeOwn = (event) => {
    const own = event.target.checked;
    setOwnOnly(own);
  };

  const [lazyParams, setLazyParams] = useState({
    first: 0,
    rows: 20,
    page: 0
  });

  const [columnSort, setColumnSort] = useState({
    sortField: null,
    sortOrder: null
  });

  const rowsPerPage = [20, 50];

  useEffect(() => {

    let requestParams = initRequestParams(lazyParams, query, columnSort);

    if (ownOnly) {
      requestParams.user = user?.loginName;
    }

    if (columnSort.sortField === "start") {
      requestParams.order_by = "TIME";
    }

    if (columnSort.sortField === "user") {
      requestParams.order_by = "USER";
    }

    if (columnSort.sortField === "ioc") {
      requestParams.order_by = "IOC_NAME";
    }

    if (columnSort.sortField === "version") {
      requestParams.order_by = "GIT_REFERENCE";
    }

    if (deploymentStatus) {
      requestParams.status = deploymentStatus;
    }

    if (jobType) {
      requestParams.type = jobType;
    }

    getOperations(requestParams);

  }, [getOperations, lazyParams, query, columnSort, user, deploymentStatus, ownOnly, jobType])

  useEffect(() => {
    setJobList(operations.operationsList);
  }, [operations, setJobList])

  const content = (
    <SearchBar search={setQuery} loading={loading} placeholder="search in IOC name, deployment comment, user or git reference">
      <JobAsyncList
        jobs={jobList} setJobs={setJobList} loading={loading}
        totalCount={operations.totalCount} lazyParams={lazyParams} setLazyParams={setLazyParams} columnSort={columnSort} setColumnSort={setColumnSort}
        rowsPerPage={rowsPerPage} />
    </SearchBar>
  );

  return (
    <Paper className={classes.paper}>
      <Grid container spacing={1} >
        <Container>
          <Grid container spacing={1} justify="space-between">
            <Grid item xs={7}>
              <Tabs
                value={selectedTab}
                onChange={handleTabChange}
                indicatorColor="primary"
                textColor="primary">
                <Tab label={<Typography variant="h5">All</Typography>} />
                <Tab label={<Typography variant="h5">Only Running</Typography>} />
                <Tab label={<Typography variant="h5">Only Finished</Typography>} />
                <Tab label={<Typography variant="h5">Only Queued</Typography>} />
              </Tabs>
            </Grid>
            <Grid item xs={3}>
              <Autocomplete
                id="combo-box-demo"
                options={[{label: "Show all"}, {label: "Only deployments", filter: "DEPLOYMENT"}, {label: "Only commands", filter: "COMMAND"}]}
                defaultValue = { {label: "Show all"} }
                getOptionLabel={(option) => option.label}
                getOptionSelected={(option, value) => option.label === value.label}
                style={{ width: "100%" }}
                renderInput={(params) => <TextField {...params} label="Job type" variant="outlined" />}
                onChange={(event, value, reason) =>  setJobType(value?.filter)} 
                autoSelect
              />
            </Grid>
            <Grid item >
              <AccessControl allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
                renderNoAccess={() => <></>} >
                <FormControlLabel className={classes.formControl}
                  control={<Switch checked={ownOnly} onChange={handleChangeOwn} />}
                  label={<Typography variant="h5">Only my jobs</Typography>}
                />
              </AccessControl>
            </Grid>
          </Grid>
        </Container>
        <Grid item xs={12} md={12}>
          {content}
        </Grid>
      </Grid>
    </Paper>
  );
}
